/********************************************************************\
 * This program is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU General Public License as   *
 * published by the Free Software Foundation; either version 2 of   *
 * the License, or (at your option) any later version.              *
 *                                                                  *
 * This program is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public License*
 * along with this program; if not, contact:                        *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
 * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 *                                                                  *
\********************************************************************/


static const gchar* sane_account_names[] =
{
    "Bank CD",
    "Brokerage Account",
    "Bond",
    "Stock",
    "Market Index",
    "Mutual Fund",
    "Checking Account",
    "Money Market",
    "Retirement",
    "Bond",
    "Stock",
    "Market Index",
    "Mutual Fund",
    "Savings Account",
    "Spouse Retirement",
    "Bond",
    "Stock",
    "Market Index",
    "Mutual Fund",
    "Currency Trading",
    "Cash in Wallet",
    "House",
    "Other Asset",
    "Vehicle",
    "Credit Card",
    "Line of Credit",
    "Education Loan",
    "Mortgage Loan",
    "Other Loan",
    "Vehicle Loan",
    "Bonus",
    "Dividend Income",
    "Gifts Received",
    "Interest Income",
    "Bond Interest",
    "CD Interest",
    "Checking Interest",
    "Money Market Interest",
    "Other Interest",
    "Savings Interest",
    "Other Income",
    "Salary",
    "Salary (Spouse)",
    "Adjustment",
    "Auto",
    "Fees",
    "Gas",
    "Parking",
    "Repair and Maintenance",
    "Bank Service Charge",
    "Books",
    "Cable",
    "Charity",
    "Childcare",
    "Clothes",
    "Commissions",
    "Computer",
    "Dining",
    "Education",
    "Entertainment",
    "Music/Movies",
    "Recreation",
    "Travel",
    "Gifts",
    "Groceries",
    "Hobbies",
    "Home Repair",
    "Insurance",
    "Auto Insurance",
    "Health Insurance",
    "Home Insurance",
    "Life Insurance",
    "Rental Insurance",
    "Interest",
    "Education Loan Interest",
    "Mortgage Interest",
    "Other Interest",
    "Vehicle Loan Interest",
    "Laundry/Dry Cleaning",
    "Medical Expenses",
    "Miscellaneous",
    "Online Services",
    "Phone",
    "Public Transportation",
    "Rent",
    "Subscriptions",
    "Supplies",
    "Taxes",
    "Federal",
    "Medicare",
    "Other Tax",
    "Property Tax",
    "Social Security",
    "State/Province",
    "Taxes (Spouse)",
    "Federal",
    "Medicare",
    "Other Tax",
    "Social Security",
    "State/Province",
    "Utilities",
    "Electric",
    "Garbage collection",
    "Gas",
    "Water",
    "Opening Balances",
    NULL,
};

static const gchar* sane_descriptions[] =
{
    "401k Income",
    "401k match 50%",
    "Cap. gain (long)",
    "Cap. gain (short)",
    "Car Repair",
    "Charity",
    "Child Care",
    "Credit Card Refund",
    "Dividends",
    "Fed Est Tax",
    "Fed Tax Withholding",
    "Insurance Rembersal",
    "Interect Earned",
    "Interest Earned",
    "Investment Interest",
    "IRA Contrib, non Deductible",
    "IRA Contrib, Non work Spouse",
    "IRA Contrib, Spouse",
    "IRA Contribution",
    "IRA Management Fee",
    "Job One",
    "Job Two",
    "Medical, Dental",
    "Medicare",
    "Medicine",
    "Mortgage Interest",
    "Opening Balance",
    "Pointe Paid",
    "Real Estate Tax",
    "Soc Sec",
    "State and Local",
    "test dup ",
    "Unemployment Comp",
    NULL,
};

static const gchar* sane_actions[] =
{
    "Dep",
    "Wthdrw",
    "Check",
    "Int",
    "Div",
    "ATM",
    "Teller",
    "POS",
    "Phone",
    "Online",
    "AutoDep",
    "Wire",
    "EFT",
    "Credit",
    "Refund",
    "Rebate",
    "Xfer",
    "DDebit",
    NULL,
};

